/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) 1997-2004 Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact adress:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.gui;

import java.awt.*;

import javax.swing.*;

import de.uni_paderborn.fujaba.gui.comp.FujabaDialog;
import de.uni_paderborn.fujaba.uml.*;


/**
 * EditCommentDialog is an editor for all UMLCommentarys. It is introduced for the Fujaba life
 * version to replace editing of comments with a text editor.
 *
 * @author    $Author: lowende $
 * @version   $Revision: 1.13 $
 */
public class EditCommentDialog extends FujabaDialog implements TextEditAction.ClosableDialog
{
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   JTextArea sourceEditor;


   /**
    * Constructor for class EditCommentDialog
    *
    * @param frame  No description provided
    */
   public EditCommentDialog (JFrame frame)
   {
      super (frame, "Comment editor", true);
      this.guiInit();
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   public void guiInit()
   {
      JPanel mainPanel = new JPanel();

      GridBagLayout gridBag = new GridBagLayout();
      GridBagConstraints constraints = new GridBagConstraints();

      mainPanel.setLayout (gridBag);

      constraints.fill = GridBagConstraints.BOTH;
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      constraints.anchor = GridBagConstraints.NORTH;
      constraints.weightx = 1.0;

      JLabel commentLabel = new JLabel ("Comment");
      constraints.insets = new Insets (0, 3, 0, 3);
      gridBag.setConstraints (commentLabel, constraints);
      mainPanel.add (commentLabel);

      sourceEditor = new JTextArea();
      sourceEditor.setToolTipText ("Edit Comment");
      sourceEditor.setColumns (30);
      sourceEditor.setRows (5);

      JScrollPane scrollPane = new JScrollPane (sourceEditor);
      constraints.weighty = 1.0;
      constraints.insets = new Insets (0, 3, 3, 3);
      gridBag.setConstraints (scrollPane, constraints);
      mainPanel.add (scrollPane);

      JPanel buttonPanel = new JPanel();
      buttonPanel.setLayout (new FlowLayout (FlowLayout.CENTER));
      JButton textEditButton = new JButton ("Open in editor");
      textEditButton.addActionListener (new TextEditAction (this, sourceEditor));
      gridBag.setConstraints (textEditButton, constraints);
      buttonPanel.add (textEditButton);
      mainPanel.add (buttonPanel);

      Container container = this.getContentPane();
      container.add (mainPanel, BorderLayout.CENTER);
      JPanel panel = this.guiPanelOkCancelHelp();
      container.add (panel, BorderLayout.SOUTH);
      this.pack();
   }


   /**
    * Get the incr attribute of the EditCommentDialog object
    *
    * @return   The incr value
    */
   public UMLIncrement getIncr()
   {
      return incr;
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private UMLIncrement incr;


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   public void parse()
   {
      if (incr == null)
      {
         incr = new UMLCommentary();
      }
      if (incr instanceof UMLCommentary)
      {
          ((UMLCommentary) incr).setText (sourceEditor.getText().trim());
         UMLProject.get().refreshDisplay();
      }
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param incr  No description provided
    */
   public void unparse (UMLIncrement incr)
   {
      if (incr instanceof UMLCommentary)
      {
         this.incr = incr;
         String commentStr =  ((UMLCommentary) incr).getText();

         if (commentStr != null)
         {
            sourceEditor.setText (commentStr);
         }
      }
   }


   /**
    * called when sourceEditor content is shown in the TextEditor
    */
   public void hideWhileEditing()
   {
      hide();
   }


   /**
    * called when the user wants to save the content of the TextEditor
    */
   public void okPressed()
   {
      actionOkButton();
   }


   /**
    * this function is called if the cancel-button is pressed
    */
   public void actionCancelButton()
   {
      super.actionCancelButton();
   }
}

/*
 * $Log: EditCommentDialog.java,v $
 * Revision 1.13  2004/11/03 10:18:03  lowende
 * Javadoc warnings removed.
 *
 */
