/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) 1997-2004 Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact adress:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.gui;

import java.awt.*;

import javax.swing.*;
import javax.swing.tree.DefaultTreeCellRenderer;

import de.uni_paderborn.fujaba.uml.UMLClass;
import de.uni_paderborn.fujaba.uml.UMLPackage;

// $Id: ImportListCellRenderer.java,v 1.10 2004/11/03 10:18:03 lowende Exp $

/**
 * ImportListCellRenderer is used to displays the imports in the right way. for packages :
 * <fullPackageName> = .* for classes : <fullClassName>
 *
 * @author    $Author: lowende $
 * @version   $Revision: 1.10 $
 */
class ImportListCellRenderer extends DefaultListCellRenderer
{
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private static Icon classIcon;
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private static Icon packageIcon;

   static
   {
      try
      {
         //Load the Icons
         classIcon =  (new DefaultTreeCellRenderer()).getLeafIcon();
         packageIcon =  (new DefaultTreeCellRenderer()).getClosedIcon();
      }
      catch (Exception e)
      {
         System.out.println ("Couldn't load images: " + e);
      }
   }


   /**
    * Get the listCellRendererComponent attribute of the ImportListCellRenderer object
    *
    * @param list          No description provided
    * @param value         No description provided
    * @param index         No description provided
    * @param isSelected    No description provided
    * @param cellHasFocus  No description provided
    * @return              The listCellRendererComponent value
    */
   public Component getListCellRendererComponent (JList list, Object value, int index,
                                                  boolean isSelected,
                                                  boolean cellHasFocus)
   {
      super.getListCellRendererComponent (list, value, index, isSelected, cellHasFocus);

      if (value instanceof UMLPackage)
      {
         setText ( ((UMLPackage) value).getFullPackageName() + ".*");
         setIcon (packageIcon);
      }
      else if (value instanceof UMLClass)
      {
         setText ( ((UMLClass) value).getFullClassName());
         setIcon (classIcon);
      }

      return this;
   }
}

/*
 * $Log: ImportListCellRenderer.java,v $
 * Revision 1.10  2004/11/03 10:18:03  lowende
 * Javadoc warnings removed.
 *
 */
