/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) 1997-2004 Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact adress:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.packagediagrams;


import de.uni_paderborn.fujaba.fsa.swing.DiagramKindComboBoxModel;
import de.uni_paderborn.fujaba.metamodel.FDiagram;
import de.uni_paderborn.fujaba.uml.UMLDiagram;
import de.uni_paderborn.fujaba.uml.UMLDiagramItem;


/**
 * UMLClass: 'DiagramUsage' A diagram usage is a placeholder for a diagram within a packagediagram.
 *
 * @author    $Author: schneider $
 * @version   $Revision: 1.33 $
 */
public class DiagramUsage extends UMLDiagramItem
{

   /**
    * Constructor for class DiagramUsage
    */
   public DiagramUsage()
   {
      super();
   }


   /**
    * Constructor for class DiagramUsage
    *
    * @param name  No description provided
    */
   public DiagramUsage (String name)
   {
      super();
      this.setName (name);
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private String name = "";


   /**
    * Get the name attribute of the DiagramUsage object
    *
    * @return   The name value
    */
   public String getName()
   {
      return this.name;
   }


   /**
    * Sets the name attribute of the DiagramUsage object
    *
    * @param strg  The new name value
    */
   public void setName (String strg)
   {
      if (strg != null && !this.name.equals (strg))
      {
         String oldName = this.name;

         UMLDiagram diagram = this.getDiagram();

         if (diagram != null && !diagram.getName().equals (strg))
         {
            diagram.setName (strg);
         }

         this.name = strg;

         firePropertyChange ("name", oldName, this.name);
      }
   }



   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private int x = -1;


   /**
    * Get the x attribute of the DiagramUsage object
    *
    * @return   The x value
    */
   public int getX()
   {
      return x;
   }


   /**
    * Sets the x attribute of the DiagramUsage object
    *
    * @param tempX  The new x value
    */
   public void setX (int tempX)
   {
      this.x = tempX;
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private int y = -1;


   /**
    * Get the y attribute of the DiagramUsage object
    *
    * @return   The y value
    */
   public int getY()
   {
      return y;
   }


   /**
    * Sets the y attribute of the DiagramUsage object
    *
    * @param tempY  The new y value
    */
   public void setY (int tempY)
   {
      this.y = tempY;
   }


   /**
    * <pre>
    *             0..1    usages    n
    * UMLDiagram --------------------- DiagramUsage
    *             diagram      usages
    * </pre>
    */
   private UMLDiagram diagram;


   /**
    * UMLMethod: '+ setDiagram (value: UMLDiagram): Boolean'
    *
    * @param value  The new diagram value
    * @return       No description provided
    */
   public boolean setDiagram (UMLDiagram value)
   {
      System.out.println ("Setting " + this + "'s diagram to " + value);
      boolean changed = false;
      if (this.diagram != value &&  (value == null || !value.isCyclic (this)))
      {
         UMLDiagram oldValue = this.diagram;
         if (this.diagram != null)
         {
            this.diagram = null;
            oldValue.removeFromUsages (this);
         }
         this.diagram = value;
         if (value != null)
         {
            value.addToUsages (this);
         }
         changed = true;

         if (value != null)
         {
            this.setName (this.diagram.getName());
            System.out.println ("value's class is " + value);
            Integer kind = (Integer) DiagramKindComboBoxModel.diagClassToKindNo.get (value.getClass());
            if (kind != null)
            {
               this.setDiagramKind (kind.intValue());
            }
         }
         firePropertyChange ("diagram", oldValue, value);
      }
      return changed;
   }


   /**
    * UMLMethod: '+ getDiagram (): UMLDiagram'
    *
    * @return   The diagram value
    */
   public UMLDiagram getDiagram()
   {
      return this.diagram;
   }


   /**
    * UMLMethod: '+ removeYou (): Void'
    */
   public void removeYou()
   {
      FDiagram tmpDiagram = getDiagram();
      if (tmpDiagram != null)
      {
         setDiagram (null);
      } // if

      super.removeYou();
   }


   /**
    * UMLAttribute: 'whichDiagram : String'
    */
   private String whichDiagram = "package";


   /**
    * UMLMethod: 'Read access method for attribute whichDiagram : String'
    *
    * @return   The whichDiagram value
    */
   public String getWhichDiagram()
   {
      return whichDiagram;
   }


   /**
    * UMLAttribute: 'diagramKind : Integer = 1'
    */
   private int diagramKind = 0;


   /**
    * UMLMethod: 'Read access method for attribute diagramKind : Integer = 1'
    *
    * @return   The diagramKind value
    */
   public int getDiagramKind()
   {
      return this.diagramKind;
   }


   /**
    * UMLMethod: 'Write access method for attribute diagramKind : Integer = 1'
    *
    * @param diagramKind  The new diagramKind value
    * @return             No description provided
    */
   public int setDiagramKind (int diagramKind)
   {
      System.out.println ("new diagram kind: " + diagramKind);

      int oldKind = this.diagramKind;

      FDiagram diagram = this.getDiagram();

      if (diagram == null)
      {
         this.diagramKind = diagramKind;
         firePropertyChange ("diagramKind", oldKind, diagramKind);
      }
      else
      {
         Integer kind = (Integer) DiagramKindComboBoxModel.diagClassToKindNo.get
             (diagram.getClass());

         if (kind != null)
         {
            int currentDiagramKind = kind.intValue();
            if (currentDiagramKind != this.diagramKind)
            {
               this.diagramKind = currentDiagramKind;
               firePropertyChange ("diagramKind", oldKind, currentDiagramKind);
            }
         }
         else
         {
            System.out.println ("This should never happen. Its Mike's fault. \n"
               + "DiagramKindComboBoxModel.diagClassToKindNo does not contain " + diagram.getClass());
         }

         if (this.diagramKind != diagramKind)
         {
            firePropertyChange ("diagramKind", diagramKind, this.diagramKind);
            System.out.println ("overridden the setdiagramkind");
         }

      }

      return this.diagramKind;
   }
}

/*
 * $Log: DiagramUsage.java,v $
 * Revision 1.33  2004/10/20 17:50:02  schneider
 * Introduction of interfaces for class diagram classes
 *
 */
