/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) 1997-2004 Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact adress:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.usecase.actions;

import java.awt.event.ActionEvent;
import java.util.Iterator;

import javax.swing.*;

import de.uni_paderborn.fujaba.app.FrameMain;
import de.uni_paderborn.fujaba.uml.UMLDiagram;
import de.uni_paderborn.fujaba.uml.UMLProject;
import de.uni_paderborn.fujaba.usecase.*;


/**
 * Create a new UsecaseUsage.
 *
 * @author    $Author: schneider $
 * @version   $Revision: 1.14 $
 */
public class NewUsecaseUsageAction extends AbstractAction
{
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param event  No description provided
    */
   public void actionPerformed (ActionEvent event)
   {
      Object source = event.getSource();

      if (source instanceof Iterator)
      {
         try
         {
            // get selected objects
            Iterator iter = (Iterator) source;
            UsecaseActor usecaseActor = null;
            Usecase usecase = null;

            // get two selected objects
            while (iter.hasNext())
            {
               Object obj = iter.next();
               if (obj instanceof Usecase)
               {
                  usecase = (Usecase) obj;
               }
               else if (obj instanceof UsecaseActor)
               {
                  usecaseActor = (UsecaseActor) obj;
               }
            }

            // an actor may use a usecase only once
            Iterator usageIterator = usecaseActor.iteratorOfUsages();
            while (usageIterator.hasNext())
            {
               UsecaseUsage usage = (UsecaseUsage) usageIterator.next();
               if (usage.getActor() == usecaseActor
                  && usage.getUsecase() == usecase)
               {
                  throw new Exception ("Error: Actor '" + usecaseActor.getName()
                     + "' already uses Use Case '"
                     + usecase.getName() + "'!");
               }
            }

            // finally create the new usage
            if (usecaseActor != null && usecase != null)
            {
               try
               {
                  UsecaseUsage usecaseUsage = new UsecaseUsage();
                  usecaseUsage.setActor (usecaseActor);
                  usecaseUsage.setUsecase (usecase);
                  UMLDiagram diagram = UMLProject.get().getCurrentUMLDiagram();
                  diagram.addToElements (usecaseUsage);

               }
               catch (Exception e)
               {
                  e.printStackTrace();
               }
               finally
               {
                  UMLProject.get().refreshDisplay();
               }
            }

         }
         catch (Exception e)
         {
            FrameMain.get().setStatusLabel (e.getMessage());
         }
      }
   } // actionPerformed

}

/*
 * $Log: NewUsecaseUsageAction.java,v $
 * Revision 1.14  2004/10/20 17:50:29  schneider
 * Introduction of interfaces for class diagram classes
 *
 */
