/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) 1997-2004 Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact adress:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.usecase.unparse;

import java.awt.*;
import java.util.Set;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.border.CompoundBorder;

import de.uni_paderborn.fujaba.fsa.FSAObject;
import de.uni_paderborn.fujaba.fsa.FSAPanel;
import de.uni_paderborn.fujaba.fsa.listener.*;
import de.uni_paderborn.fujaba.fsa.swing.border.*;
import de.uni_paderborn.fujaba.fsa.unparse.AbstractUnparseModule;
import de.uni_paderborn.fujaba.fsa.unparse.LogicUnparseInterface;
import de.uni_paderborn.fujaba.usecase.UsecaseSystem;


/**
 * Class for displaying a {@link UsecaseSystem} on screen. A UsecaseSystem is the System-box
 * of a {@link de.uni_paderborn.fujaba.usecase.UsecaseDiagram}. Inside of it is a {@link de.uni_paderborn.fujaba.usecase.UsecaseSystemDiagram}
 * to hold all the {@link de.uni_paderborn.fujaba.usecase.Usecase}s.
 *
 * @author    $Author: schneider $
 * @version   $Revision: 1.22 $
 * @see       de.uni_paderborn.fujaba.usecase.Usecase
 * @see       de.uni_paderborn.fujaba.usecase.UsecaseDiagram
 * @see       UsecaseSystem
 * @see       de.uni_paderborn.fujaba.usecase.UsecaseSystemDiagram
 */
public class UMUsecaseSystem extends AbstractUnparseModule
{
   /**
    * Get the childProperties attribute of the UMUsecaseSystem object
    *
    * @param props  No description provided
    */
   public void getChildProperties (Set props)
   {
      super.getChildProperties (props);
      props.add ("systemDiagram");
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param parent  No description provided
    * @param incr    No description provided
    * @return        No description provided
    */
   public FSAObject create (FSAObject parent, LogicUnparseInterface incr)
   {
      UsecaseSystem usecaseSystem = (UsecaseSystem) incr;

      FSAPanel panel = new FSAPanel (incr, getMainFsaName(), parent.getJComponent());
      Border lineBorder = new LineBorder (FSAObject.COLOR_FOREGROUND);
      Border spaceBorder = new EmptyBorder (13, 13, 13, 13);
      TextBorder textBorder =
         new TextBorder (FSAObject.COLOR_FOREGROUND, usecaseSystem.getName());
      textBorder.setFont (textBorder.getFont().deriveFont (Font.BOLD));
      Border compoundBorder = new CompoundBorder (lineBorder, spaceBorder);
      panel.setBorder (new CompoundBorder (compoundBorder, textBorder));
      panel.setOpaque (false);

      // add mouse listener for moving and selecting
      JComponent jPanel = panel.getJComponent();
      AscendDescendMouseHandler.addMouseInputListener (jPanel, SelectionMouseListener.get());
      AscendDescendMouseHandler.addMouseInputListener (jPanel, DragMouseListener.get());
      AscendDescendMouseHandler.addMouseInputListener (jPanel, AscendDescendMouseHandler.getDescendConsumer());
      SelectionListenerHelper.addSelectionListener (jPanel, BorderHighlighter.get());

      return panel;
   }

}

/*
 * $Log: UMUsecaseSystem.java,v $
 * Revision 1.22  2004/10/20 17:50:29  schneider
 * Introduction of interfaces for class diagram classes
 *
 */
