/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) 1997-2004 Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact adress:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.usecase.unparse;

import javax.swing.*;

import de.uni_paderborn.fujaba.fsa.*;
import de.uni_paderborn.fujaba.fsa.listener.*;
import de.uni_paderborn.fujaba.fsa.swing.*;
import de.uni_paderborn.fujaba.fsa.unparse.AbstractUnparseModule;
import de.uni_paderborn.fujaba.fsa.unparse.LogicUnparseInterface;
import de.uni_paderborn.fujaba.usecase.*;


/**
 * Class for displaying a {@link UsecaseUsage} on screen. A UsecaseUsage is the arrow pointing
 * from a {@link UsecaseActor} a to a {@link Usecase} u indicating that this Actor a uses that
 * Usecase u.
 *
 * @author    $Author: schneider $
 * @version   $Revision: 1.19 $
 * @see       UsecaseUsage
 * @see       UsecaseActor
 * @see       Usecase
 */
public class UMUsecaseUsage extends AbstractUnparseModule
{
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private final static int LAYER = JLayeredPane.DRAG_LAYER.intValue() + 20;
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private final static int DRAG_LAYER = LAYER + 1;
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private final static String LAYER_NAME = "UseCaseUsageLayer";

   static
   {
      LayerManager.get().addToLayerPositions (LAYER_NAME, new Integer (LAYER));
      LayerManager.get().addToLayerPositions (LayerManager.DRAG_LAYER_PREFIX + LAYER_NAME, new Integer (DRAG_LAYER));
   }


   /**
    * Get the mainFsaName attribute of the UMUsecaseUsage object
    *
    * @return   The mainFsaName value
    */
   public String getMainFsaName()
   {
      return "line";
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param parent  No description provided
    * @param incr    No description provided
    * @return        No description provided
    */
   public FSAObject create (FSAObject parent, LogicUnparseInterface incr)
   {
      FSAContainer container = (FSAContainer) parent;
      UsecaseUsage usage = (UsecaseUsage) incr;

      UsecaseDiagram usecaseDiagram = (UsecaseDiagram) container.getLogic();
      UsecaseSystem system = usecaseDiagram.getUsecaseSystem();
      UsecaseSystemDiagram systemDiagram = system.getSystemDiagram();

      // the line representing the usage
      FSABendLine line = new FSABendLine (usage, getMainFsaName(), parent.getJComponent(), new JArrowedBendLine());

      UsecaseActor actor = usage.getActor();
      Usecase usecase = usage.getUsecase();

      // This was the starting motivation for the layer-stuff:
      // The usage must be drawn in some kind of "higher layer", or it will
      // be overdrawn by the UsecaseSystem as soon as this is selected.

      // place in a layer relative to other Use Case Diagram components
      LayerManager.setLayerType (line.getJComponent(), LAYER_NAME);

      // the Actor's main panel
      FSAObject actorFSA = actor != null ? actor.getFromFsaObjects (container.getID() + ".entry") : null;

      // the Usecase's main panel (Usecases are placed in the
      // System-Box, thus the different container)
      FSAObject systemContainer = system.getFromFsaObjects (container.getID() + ".entry");
      FSAObject systemDiagContainer = systemDiagram.getFromFsaObjects (systemContainer.getID() + ".entry");

      FSAObject usecaseFSA = usecase != null ? usecase.getFromFsaObjects (systemDiagContainer.getID() + ".entry") : null;

      FSAGrab grab = new FSAGrab (incr, "usecase", parent.getJComponent());
      ResizeableGrabUI ui = (ResizeableGrabUI) ResizeableGrabUI.createUI (grab.getJComponent());
      grab.setUI (ui);
      grab.setTarget (usecaseFSA);
      grab.getJComponent().setOpaque (false);

      line.setEndBend (grab);

      grab = new FSAGrab (incr, "actor", parent.getJComponent());
      ui = (ResizeableGrabUI) ResizeableGrabUI.createUI (grab.getJComponent());
      grab.setUI (ui);
      grab.setTarget (actorFSA);
      grab.getJComponent().setOpaque (false);

      line.setStartBend (grab);

      JComponent jPanel = line.getJComponent();
      AscendDescendMouseHandler.addMouseInputListener (jPanel, SelectionMouseListener.get());
      SelectionListenerHelper.addSelectionListener (jPanel, ForegroundHighlighter.get());

      return line;
   }
}

/*
 * $Log: UMUsecaseUsage.java,v $
 * Revision 1.19  2004/10/20 17:50:29  schneider
 * Introduction of interfaces for class diagram classes
 *
 */
